@@ -28,6 +28,7 @@ import io.grpc.stub.StreamObserver;
 import lombok.AccessLevel;
 import lombok.Getter;
 import lombok.extern.slf4j.Slf4j;
+import org.apache.skywalking.banyandb.common.v1.BanyandbCommon;
 import org.apache.skywalking.banyandb.measure.v1.BanyandbMeasure;
 import org.apache.skywalking.banyandb.measure.v1.MeasureServiceGrpc;
 import org.apache.skywalking.banyandb.stream.v1.BanyandbStream;
@@ -36,6 +37,8 @@ import org.apache.skywalking.banyandb.v1.client.grpc.HandleExceptionsWith;
 import org.apache.skywalking.banyandb.v1.client.grpc.channel.ChannelManager;
 import org.apache.skywalking.banyandb.v1.client.grpc.channel.DefaultChannelFactory;
 import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException;
+import org.apache.skywalking.banyandb.v1.client.grpc.exception.InternalException;
+import org.apache.skywalking.banyandb.v1.client.grpc.exception.InvalidArgumentException;
 import org.apache.skywalking.banyandb.v1.client.metadata.Group;
 import org.apache.skywalking.banyandb.v1.client.metadata.GroupMetadataRegistry;
 import org.apache.skywalking.banyandb.v1.client.metadata.IndexRule;
@@ -50,12 +53,17 @@ import org.apache.skywalking.banyandb.v1.client.metadata.PropertyStore;
 import org.apache.skywalking.banyandb.v1.client.metadata.ResourceExist;
 import org.apache.skywalking.banyandb.v1.client.metadata.Stream;
 import org.apache.skywalking.banyandb.v1.client.metadata.StreamMetadataRegistry;
+import org.apache.skywalking.banyandb.v1.client.metadata.TopNAggregation;
+import org.apache.skywalking.banyandb.v1.client.metadata.TopNAggregationMetadataRegistry;
 
 import java.io.Closeable;
 import java.io.IOException;
+import java.net.URI;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collections;
 import java.util.List;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.stream.Collectors;
@@ -76,14 +84,8 @@ import static com.google.common.base.Preconditions.checkState;
  */
 @Slf4j
 public class BanyanDBClient implements Closeable {
-    /**
-     * The hostname of BanyanDB server.
-     */
-    private final String host;
-    /**
-     * The port of BanyanDB server.
-     */
-    private final int port;
+
+    private final String[] targets;
     /**
      * Options for server connection.
      */
@@ -123,28 +125,35 @@ public class BanyanDBClient implements Closeable {
      */
     private final ReentrantLock connectionEstablishLock;
 
+    /**
+     * Client local metadata cache.
+     */
+    private final MetadataCache metadataCache;
+
     /**
      * Create a BanyanDB client instance with a default options.
      *
-     * @param host IP or domain name
-     * @param port Server port
+     * @param targets server targets
      */
-    public BanyanDBClient(String host, int port) {
-        this(host, port, new Options());
+    public BanyanDBClient(String... targets) {
+        this(targets, new Options());
     }
 
     /**
      * Create a BanyanDB client instance with a customized options.
      *
-     * @param host    IP or domain name
-     * @param port    Server port
+     * @param targets server targets
      * @param options customized options
      */
-    public BanyanDBClient(String host, int port, Options options) {
-        this.host = host;
-        this.port = port;
+    public BanyanDBClient(String[] targets, Options options) {
+        String[] tt = Preconditions.checkNotNull(targets, "targets");
+        checkState(tt.length > 0, "targets' size must be more than 1");
+        tt = Arrays.stream(tt).filter(t -> !Strings.isNullOrEmpty(t)).toArray(size -> new String[size]);
+        checkState(tt.length > 0, "valid targets' size must be more than 1");
+        this.targets = tt;
         this.options = options;
         this.connectionEstablishLock = new ReentrantLock();
+        this.metadataCache = new MetadataCache();
     }
 
     /**
@@ -156,8 +165,12 @@ public class BanyanDBClient implements Closeable {
         connectionEstablishLock.lock();
         try {
             if (!isConnected) {
+                URI[] addresses = new URI[this.targets.length];
+                for (int i = 0; i < this.targets.length; i++) {
+                        addresses[i] = URI.create("//" + this.targets[i]);
+                }
                 this.channel = ChannelManager.create(this.options.buildChannelManagerSettings(),
-                        new DefaultChannelFactory(this.host, this.port, this.options));
+                        new DefaultChannelFactory(addresses, this.options));
                 streamServiceBlockingStub = StreamServiceGrpc.newBlockingStub(this.channel);
                 measureServiceBlockingStub = MeasureServiceGrpc.newBlockingStub(this.channel);
                 streamServiceStub = StreamServiceGrpc.newStub(this.channel);
@@ -190,26 +203,65 @@ public class BanyanDBClient implements Closeable {
      * Perform a single write with given entity.
      *
      * @param streamWrite the entity to be written
+     * @return a future of write result
      */
-    public void write(StreamWrite streamWrite) {
+    public CompletableFuture<Void> write(StreamWrite streamWrite) {
         checkState(this.streamServiceStub != null, "stream service is null");
 
+        CompletableFuture<Void> future = new CompletableFuture<>();
         final StreamObserver<BanyandbStream.WriteRequest> writeRequestStreamObserver
                 = this.streamServiceStub
                 .withDeadlineAfter(this.getOptions().getDeadline(), TimeUnit.SECONDS)
                 .write(
                         new StreamObserver<BanyandbStream.WriteResponse>() {
+                            private BanyanDBException responseException;
+
                             @Override
                             public void onNext(BanyandbStream.WriteResponse writeResponse) {
+                                switch (writeResponse.getStatus()) {
+                                    case STATUS_INVALID_TIMESTAMP:
+                                        responseException = new InvalidArgumentException(
+                                                "Invalid timestamp: " + streamWrite.getTimestamp(), null, Status.Code.INVALID_ARGUMENT, false);
+                                        break;
+                                    case STATUS_NOT_FOUND:
+                                        responseException = new InvalidArgumentException(
+                                                "Invalid metadata: " + streamWrite.entityMetadata, null, Status.Code.INVALID_ARGUMENT, false);
+                                        break;
+                                    case STATUS_EXPIRED_SCHEMA:
+                                        BanyandbCommon.Metadata metadata = writeResponse.getMetadata();
+                                        log.warn("The schema {}.{} is expired, trying update the schema...",
+                                                metadata.getGroup(), metadata.getName());
+                                        try {
+                                            BanyanDBClient.this.findStream(metadata.getGroup(), metadata.getName());
+                                        } catch (BanyanDBException e) {
+                                            String warnMessage = String.format("Failed to refresh the stream schema %s.%s",
+                                                    metadata.getGroup(), metadata.getName());
+                                            log.warn(warnMessage, e);
+                                        }
+                                        responseException = new InvalidArgumentException(
+                                                "Expired revision: " + metadata.getModRevision(), null, Status.Code.INVALID_ARGUMENT, true);
+                                        break;
+                                    case STATUS_INTERNAL_ERROR:
+                                        responseException = new InternalException(
+                                                "Internal error occurs in server", null, Status.Code.INTERNAL, true);
+                                        break;
+                                    default:
+                                }
                             }
 
                             @Override
                             public void onError(Throwable throwable) {
                                 log.error("Error occurs in flushing streams.", throwable);
+                                future.completeExceptionally(throwable);
                             }
 
                             @Override
                             public void onCompleted() {
+                                if (responseException == null) {
+                                    future.complete(null);
+                                } else {
+                                    future.completeExceptionally(responseException);
+                                }
                             }
                         });
         try {
@@ -217,6 +269,7 @@ public class BanyanDBClient implements Closeable {
         } finally {
             writeRequestStreamObserver.onCompleted();
         }
+        return future;
     }
 
     /**
@@ -231,7 +284,7 @@ public class BanyanDBClient implements Closeable {
     public StreamBulkWriteProcessor buildStreamWriteProcessor(int maxBulkSize, int flushInterval, int concurrency) {
         checkState(this.streamServiceStub != null, "stream service is null");
 
-        return new StreamBulkWriteProcessor(this.streamServiceStub, maxBulkSize, flushInterval, concurrency);
+        return new StreamBulkWriteProcessor(this, maxBulkSize, flushInterval, concurrency);
     }
 
     /**
@@ -246,7 +299,44 @@ public class BanyanDBClient implements Closeable {
     public MeasureBulkWriteProcessor buildMeasureWriteProcessor(int maxBulkSize, int flushInterval, int concurrency) {
         checkState(this.measureServiceStub != null, "measure service is null");
 
-        return new MeasureBulkWriteProcessor(this.measureServiceStub, maxBulkSize, flushInterval, concurrency);
+        return new MeasureBulkWriteProcessor(this, maxBulkSize, flushInterval, concurrency);
+    }
+
+    /**
+     * Build a MeasureWrite request.
+     *
+     * @param group     the group of the measure
+     * @param name      the name of the measure
+     * @param timestamp the timestamp of the measure
+     * @return the request to be built
+     */
+    public MeasureWrite createMeasureWrite(String group, String name, long timestamp) {
+        return new MeasureWrite(this.metadataCache.findMetadata(group, name), timestamp);
+    }
+
+    /**
+     * Build a StreamWrite request.
+     *
+     * @param group     the group of the stream
+     * @param name      the name of the stream
+     * @param elementId the primary key of the stream
+     * @return the request to be built
+     */
+    public StreamWrite createStreamWrite(String group, String name, final String elementId) {
+        return new StreamWrite(this.metadataCache.findMetadata(group, name), elementId);
+    }
+
+    /**
+     * Build a StreamWrite request.
+     *
+     * @param group     the group of the stream
+     * @param name      the name of the stream
+     * @param elementId the primary key of the stream
+     * @param timestamp the timestamp of the stream
+     * @return the request to be built
+     */
+    public StreamWrite createStreamWrite(String group, String name, final String elementId, long timestamp) {
+        return new StreamWrite(this.metadataCache.findMetadata(group, name), elementId, timestamp);
     }
 
     /**
@@ -261,10 +351,26 @@ public class BanyanDBClient implements Closeable {
         final BanyandbStream.QueryResponse response = HandleExceptionsWith.callAndTranslateApiException(() ->
                 this.streamServiceBlockingStub
                         .withDeadlineAfter(this.getOptions().getDeadline(), TimeUnit.SECONDS)
-                        .query(streamQuery.build()));
+                        .query(streamQuery.build(this.metadataCache.findMetadata(streamQuery.group, streamQuery.name))));
         return new StreamQueryResponse(response);
     }
 
+    /**
+     * Query TopN according to given conditions
+     *
+     * @param topNQuery condition for query
+     * @return hint topN.
+     */
+    public TopNQueryResponse query(TopNQuery topNQuery) throws BanyanDBException {
+        checkState(this.measureServiceStub != null, "measure service is null");
+
+        final BanyandbMeasure.TopNResponse response = HandleExceptionsWith.callAndTranslateApiException(() ->
+                this.measureServiceBlockingStub
+                        .withDeadlineAfter(this.getOptions().getDeadline(), TimeUnit.SECONDS)
+                        .topN(topNQuery.build()));
+        return new TopNQueryResponse(response);
+    }
+
     /**
      * Query measures according to given conditions
      *
@@ -277,7 +383,7 @@ public class BanyanDBClient implements Closeable {
         final BanyandbMeasure.QueryResponse response = HandleExceptionsWith.callAndTranslateApiException(() ->
                 this.measureServiceBlockingStub
                         .withDeadlineAfter(this.getOptions().getDeadline(), TimeUnit.SECONDS)
-                        .query(measureQuery.build()));
+                        .query(measureQuery.build(this.metadataCache.findMetadata(measureQuery.group, measureQuery.name))));
         return new MeasureQueryResponse(response);
     }
 
@@ -300,9 +406,26 @@ public class BanyanDBClient implements Closeable {
      */
     public void define(Stream stream) throws BanyanDBException {
         StreamMetadataRegistry streamRegistry = new StreamMetadataRegistry(checkNotNull(this.channel));
-        streamRegistry.create(stream);
+        long modRevision = streamRegistry.create(stream);
         defineIndexRules(stream, stream.indexRules());
-        MetadataCache.INSTANCE.register(stream);
+
+        stream = stream.withModRevision(modRevision);
+        this.metadataCache.register(stream);
+    }
+
+    /**
+     * Delete a stream
+     *
+     * @param stream the stream to be deleted
+     * @return true if the stream is deleted successfully
+     */
+    public boolean delete(Stream stream) throws BanyanDBException {
+        StreamMetadataRegistry streamRegistry = new StreamMetadataRegistry(checkNotNull(this.channel));
+        if (streamRegistry.delete(stream.group(), stream.name())) {
+            this.metadataCache.unregister(stream);
+            return true;
+        }
+        return false;
     }
 
     /**
@@ -312,9 +435,36 @@ public class BanyanDBClient implements Closeable {
      */
     public void define(Measure measure) throws BanyanDBException {
         MeasureMetadataRegistry measureRegistry = new MeasureMetadataRegistry(checkNotNull(this.channel));
-        measureRegistry.create(measure);
+        long modRevision = measureRegistry.create(measure);
         defineIndexRules(measure, measure.indexRules());
-        MetadataCache.INSTANCE.register(measure);
+
+        measure = measure.withModRevision(modRevision);
+        this.metadataCache.register(measure);
+    }
+
+    /**
+     * Delete a measure
+     *
+     * @param measure the measure to be deleted
+     * @return true if the measure is deleted successfully
+     */
+    public boolean delete(Measure measure) throws BanyanDBException {
+        MeasureMetadataRegistry measureRegistry = new MeasureMetadataRegistry(checkNotNull(this.channel));
+        if (measureRegistry.delete(measure.group(), measure.name())) {
+            this.metadataCache.unregister(measure);
+            return true;
+        }
+        return false;
+    }
+
+    /**
+     * Define a new TopNAggregation
+     *
+     * @param topNAggregation the topN rule to be created
+     */
+    public void define(TopNAggregation topNAggregation) throws BanyanDBException {
+        TopNAggregationMetadataRegistry registry = new TopNAggregationMetadataRegistry(checkNotNull(this.channel));
+        registry.create(topNAggregation);
     }
 
     /**
@@ -361,8 +511,22 @@ public class BanyanDBClient implements Closeable {
      * @return all properties belonging to the group and the name
      */
     public List<Property> findProperties(String group, String name) throws BanyanDBException {
+        return findProperties(group, name, null, null);
+
+    }
+
+    /**
+     * List Properties
+     *
+     * @param group group of the metadata
+     * @param name  name of the metadata
+     * @param ids   identities of the properties
+     * @param tags  tags to be returned
+     * @return all properties belonging to the group and the name
+     */
+    public List<Property> findProperties(String group, String name, List<String> ids, List<String> tags) throws BanyanDBException {
         PropertyStore store = new PropertyStore(checkNotNull(this.channel));
-        return store.list(group, name);
+        return store.list(group, name, ids, tags);
     }
 
     /**
@@ -380,6 +544,16 @@ public class BanyanDBClient implements Closeable {
         return store.delete(group, name, id, tags);
     }
 
+    /**
+     * Keep alive the property
+     *
+     * @param leaseId lease id of the property
+     */
+    public void keepAliveProperty(long leaseId) throws BanyanDBException {
+        PropertyStore store = new PropertyStore(checkNotNull(this.channel));
+        store.keepAlive(leaseId);
+    }
+
     /**
      * Bind index rule to the stream
      *
@@ -400,6 +574,9 @@ public class BanyanDBClient implements Closeable {
                 throw ex;
             }
         }
+        if (indexRules.isEmpty()) {
+            return;
+        }
 
         List<String> indexRuleNames = indexRules.stream().map(IndexRule::name).collect(Collectors.toList());
 
@@ -433,6 +610,9 @@ public class BanyanDBClient implements Closeable {
                 throw ex;
             }
         }
+        if (indexRules.isEmpty()) {
+            return;
+        }
 
         List<String> indexRuleNames = indexRules.stream().map(IndexRule::name).collect(Collectors.toList());
 
@@ -464,6 +644,20 @@ public class BanyanDBClient implements Closeable {
         }
     }
 
+    /**
+     * Try to find the TopNAggregation from the BanyanDB with given group and name.
+     *
+     * @param group group of the TopNAggregation
+     * @param name  name of the TopNAggregation
+     * @return TopNAggregation if found. Otherwise, null is returned.
+     */
+    public TopNAggregation findTopNAggregation(String group, String name) throws BanyanDBException {
+        Preconditions.checkArgument(!Strings.isNullOrEmpty(group));
+        Preconditions.checkArgument(!Strings.isNullOrEmpty(name));
+
+        return new TopNAggregationMetadataRegistry(checkNotNull(this.channel)).get(group, name);
+    }
+
     /**
      * Try to find the stream from the BanyanDB with given group and name.
      *
@@ -477,7 +671,7 @@ public class BanyanDBClient implements Closeable {
 
         Stream s = new StreamMetadataRegistry(checkNotNull(this.channel)).get(group, name);
         s = s.withIndexRules(findIndexRulesByGroupAndBindingName(group, IndexRuleBinding.defaultBindingRule(name)));
-        MetadataCache.INSTANCE.register(s);
+        this.metadataCache.register(s);
         return s;
     }
 
@@ -494,7 +688,7 @@ public class BanyanDBClient implements Closeable {
 
         Measure m = new MeasureMetadataRegistry(checkNotNull(this.channel)).get(group, name);
         m = m.withIndexRules(findIndexRulesByGroupAndBindingName(group, IndexRuleBinding.defaultBindingRule(name)));
-        MetadataCache.INSTANCE.register(m);
+        this.metadataCache.register(m);
         return m;
     }
 
@@ -502,7 +696,16 @@ public class BanyanDBClient implements Closeable {
             BanyanDBException {
         IndexRuleBindingMetadataRegistry irbRegistry = new IndexRuleBindingMetadataRegistry(checkNotNull(this.channel));
 
-        IndexRuleBinding irb = irbRegistry.get(group, bindingName);
+        IndexRuleBinding irb;
+        try {
+            irb = irbRegistry.get(group, bindingName);
+        } catch (BanyanDBException ex) {
+            if (ex.getStatus().equals(Status.Code.NOT_FOUND)) {
+                return Collections.emptyList();
+            }
+            throw ex;
+        }
+
         if (irb == null) {
             return Collections.emptyList();
         }
@@ -510,7 +713,14 @@ public class BanyanDBClient implements Closeable {
         IndexRuleMetadataRegistry irRegistry = new IndexRuleMetadataRegistry(checkNotNull(this.channel));
         List<IndexRule> indexRules = new ArrayList<>(irb.rules().size());
         for (final String rule : irb.rules()) {
-            indexRules.add(irRegistry.get(group, rule));
+            try {
+                indexRules.add(irRegistry.get(group, rule));
+            } catch (BanyanDBException ex) {
+                if (ex.getStatus().equals(Status.Code.NOT_FOUND)) {
+                    continue;
+                }
+                throw ex;
+            }
         }
         return indexRules;
     }
@@ -543,6 +753,20 @@ public class BanyanDBClient implements Closeable {
         return new MeasureMetadataRegistry(checkNotNull(this.channel)).exist(group, name);
     }
 
+    /**
+     * Check if the given TopNAggregation exists.
+     *
+     * @param group group of the TopNAggregation
+     * @param name  name of the TopNAggregation
+     * @return ResourceExist which indicates whether group and TopNAggregation exist
+     */
+    public ResourceExist existTopNAggregation(String group, String name) throws BanyanDBException {
+        Preconditions.checkArgument(!Strings.isNullOrEmpty(group));
+        Preconditions.checkArgument(!Strings.isNullOrEmpty(name));
+
+        return new TopNAggregationMetadataRegistry(checkNotNull(this.channel)).exist(group, name);
+    }
+
     @Override
     public void close() throws IOException {
         connectionEstablishLock.lock();
