@@ -20,13 +20,12 @@ package org.apache.skywalking.banyandb.v1.client;
 
 import com.google.common.base.Preconditions;
 import com.google.common.base.Strings;
-import com.google.common.collect.ImmutableSet;
 import lombok.RequiredArgsConstructor;
 import lombok.Setter;
 import org.apache.skywalking.banyandb.measure.v1.BanyandbMeasure;
 import org.apache.skywalking.banyandb.model.v1.BanyandbModel;
 import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException;
-import org.apache.skywalking.banyandb.v1.client.metadata.Measure;
+import org.apache.skywalking.banyandb.v1.client.metadata.MetadataCache;
 
 import java.util.Set;
 
@@ -55,7 +54,7 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
     }
 
     public MeasureQuery(final String group, final String name, final TimestampRange timestampRange, final Set<String> tagProjections, final Set<String> fieldProjections) {
-        super(group, name, timestampRange, addIDProjection(tagProjections));
+        super(group, name, timestampRange, tagProjections);
         this.fieldProjections = fieldProjections;
     }
 
@@ -144,7 +143,11 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
     /**
      * @return QueryRequest for gRPC level query.
      */
-    BanyandbMeasure.QueryRequest build() throws BanyanDBException {
+    @Override
+    BanyandbMeasure.QueryRequest build(MetadataCache.EntityMetadata entityMetadata) throws BanyanDBException {
+        if (entityMetadata == null) {
+            throw new IllegalArgumentException("entity metadata is null");
+        }
         final BanyandbMeasure.QueryRequest.Builder builder = BanyandbMeasure.QueryRequest.newBuilder();
         builder.setMetadata(buildMetadata());
         if (timestampRange != null) {
@@ -152,13 +155,18 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
         } else {
             builder.setTimeRange(TimestampRange.MAX_RANGE);
         }
-        builder.setTagProjection(buildTagProjections());
-        builder.setFieldProjection(BanyandbMeasure.QueryRequest.FieldProjection.newBuilder()
-                .addAllNames(fieldProjections)
-                .build());
+        BanyandbModel.TagProjection tagProjections = buildTagProjections(entityMetadata);
+        if (tagProjections.getTagFamiliesCount() > 0) {
+            builder.setTagProjection(buildTagProjections(entityMetadata));
+        }
+        if (!fieldProjections.isEmpty()) {
+            builder.setFieldProjection(BanyandbMeasure.QueryRequest.FieldProjection.newBuilder()
+                    .addAllNames(fieldProjections)
+                    .build());
+        }
         if (this.aggregation != null) {
             BanyandbMeasure.QueryRequest.GroupBy.Builder groupByBuilder = BanyandbMeasure.QueryRequest.GroupBy.newBuilder()
-                    .setTagProjection(buildTagProjections(this.aggregation.groupByTagsProjection));
+                    .setTagProjection(buildTagProjections(entityMetadata, this.aggregation.groupByTagsProjection));
             if (Strings.isNullOrEmpty(this.aggregation.fieldName)) {
                 if (this.aggregation.aggregationType != Aggregation.Type.UNSPECIFIED) {
                     throw new IllegalArgumentException("field name cannot be null or empty if aggregation is specified");
@@ -214,12 +222,7 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
             MIN(BanyandbModel.AggregationFunction.AGGREGATION_FUNCTION_MIN),
             COUNT(BanyandbModel.AggregationFunction.AGGREGATION_FUNCTION_COUNT),
             SUM(BanyandbModel.AggregationFunction.AGGREGATION_FUNCTION_SUM);
-            private final BanyandbModel.AggregationFunction function;
+            final BanyandbModel.AggregationFunction function;
         }
     }
-
-    static ImmutableSet<String> addIDProjection(Set<String> tagProjections) {
-        // make a defensive copy in case the original one is immutable
-        return ImmutableSet.<String>builder().addAll(tagProjections).add(Measure.ID).build();
-    }
 }
