@@ -23,6 +23,7 @@ import java.util.Set;
 import lombok.Setter;
 import org.apache.skywalking.banyandb.stream.v1.BanyandbStream;
 import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException;
+import org.apache.skywalking.banyandb.v1.client.metadata.MetadataCache;
 
 /**
  * StreamQuery is the high-level query API for the stream model.
@@ -58,15 +59,22 @@ public class StreamQuery extends AbstractQuery<BanyandbStream.QueryRequest> {
     }
 
     @Override
-    BanyandbStream.QueryRequest build() throws BanyanDBException {
+    public StreamQuery or(PairQueryCondition<?> condition) {
+        return (StreamQuery) super.or(condition);
+    }
+
+    @Override
+    BanyandbStream.QueryRequest build(MetadataCache.EntityMetadata entityMetadata) throws BanyanDBException {
+        if (entityMetadata == null) {
+            throw new IllegalArgumentException("entity metadata is null");
+        }
         final BanyandbStream.QueryRequest.Builder builder = BanyandbStream.QueryRequest.newBuilder()
                 .setMetadata(buildMetadata());
         if (timestampRange != null) {
             builder.setTimeRange(timestampRange.build());
         }
-        builder.setProjection(buildTagProjections());
-        // set conditions grouped by tagFamilyName
-        builder.addAllCriteria(buildCriteria());
+        builder.setProjection(buildTagProjections(entityMetadata));
+        buildCriteria().ifPresent(builder::setCriteria);
         builder.setOffset(offset);
         builder.setLimit(limit);
         if (orderBy != null) {
