@@ -20,10 +20,15 @@ package org.apache.skywalking.banyandb.v1.client.metadata;
 
 import com.google.auto.value.AutoValue;
 import com.google.common.collect.ImmutableList;
+
+import javax.annotation.Nullable;
 import lombok.EqualsAndHashCode;
 import lombok.Getter;
 import lombok.RequiredArgsConstructor;
+
+import org.apache.skywalking.banyandb.common.v1.BanyandbCommon;
 import org.apache.skywalking.banyandb.database.v1.BanyandbDatabase;
+import org.apache.skywalking.banyandb.v1.client.util.IgnoreHashEquals;
 import org.apache.skywalking.banyandb.v1.client.util.TimeUtils;
 
 import java.time.ZonedDateTime;
@@ -32,7 +37,6 @@ import java.util.List;
 
 @AutoValue
 public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
-    public static final String ID = "id";
 
     /**
      * specs of tag families
@@ -54,6 +58,14 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
      */
     abstract Duration interval();
 
+    /**
+     * last updated revision
+     * This field can only be set by the server.
+     */
+    @Nullable
+    @IgnoreHashEquals
+    abstract Long modRevision();
+
     /**
      * index rules bound to the stream
      */
@@ -66,6 +78,10 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
                 .build();
     }
 
+    public final Measure withModRevision(Long modRevision) {
+        return toBuilder().setModRevision(modRevision).build();
+    }
+
     public static Measure.Builder create(String group, String name, Duration interval) {
         return new AutoValue_Measure.Builder().setGroup(group).setName(name).setInterval(interval);
     }
@@ -82,6 +98,8 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
 
         abstract Measure.Builder setUpdatedAt(ZonedDateTime updatedAt);
 
+        abstract Measure.Builder setModRevision(Long modRevision);
+
         abstract ImmutableList.Builder<TagFamilySpec> tagFamiliesBuilder();
 
         public final Measure.Builder addTagFamily(TagFamilySpec tagFamilySpec) {
@@ -128,6 +146,7 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
         final Measure.Builder m = Measure.create(pb.getMetadata().getGroup(), pb.getMetadata().getName(),
                         Duration.parse(pb.getInterval()))
                 .setUpdatedAt(TimeUtils.parseTimestamp(pb.getUpdatedAt()))
+                .setModRevision(pb.getMetadata().getModRevision())
                 .setEntityRelativeTags(pb.getEntity().getTagNamesList());
 
         // build tag family spec
@@ -155,9 +174,13 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
             fs.add(spec.serialize());
         }
 
+        BanyandbCommon.Metadata metadata = buildMetadata();
+        if (this.modRevision() != null) {
+            metadata = buildMetadata(this.modRevision());
+        }
         BanyandbDatabase.Measure.Builder b = BanyandbDatabase.Measure.newBuilder()
                 .setInterval(interval().format())
-                .setMetadata(buildMetadata())
+                .setMetadata(metadata)
                 .addAllTagFamilies(tfs)
                 .addAllFields(fs)
                 .setEntity(BanyandbDatabase.Entity.newBuilder().addAllTagNames(entityRelativeTags()).build());
@@ -201,7 +224,8 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
             UNSPECIFIED(BanyandbDatabase.FieldType.FIELD_TYPE_UNSPECIFIED),
             STRING(BanyandbDatabase.FieldType.FIELD_TYPE_STRING),
             INT(BanyandbDatabase.FieldType.FIELD_TYPE_INT),
-            BINARY(BanyandbDatabase.FieldType.FIELD_TYPE_DATA_BINARY);
+            BINARY(BanyandbDatabase.FieldType.FIELD_TYPE_DATA_BINARY),
+            FLOAT(BanyandbDatabase.FieldType.FIELD_TYPE_FLOAT);
 
             private final BanyandbDatabase.FieldType fieldType;
         }
@@ -244,6 +268,9 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
                 case FIELD_TYPE_DATA_BINARY:
                     b = newBinaryField(pb.getName());
                     break;
+                case FIELD_TYPE_FLOAT:
+                    b = newFloatField(pb.getName());
+                    break;
                 default:
                     throw new IllegalArgumentException("unrecognized field type");
             }
@@ -263,6 +290,15 @@ public abstract class Measure extends NamedSchema<BanyandbDatabase.Measure> {
             return b.build();
         }
 
+        /**
+         * Create a builder with float type
+         *
+         * @param name name of the field
+         */
+        public static Builder newFloatField(String name) {
+            return new Builder(name, FieldType.FLOAT);
+        }
+
         /**
          * Create a builder with string type
          *
