@@ -36,11 +36,12 @@ public class MeasureMetadataRegistry extends MetadataClient<MeasureRegistryServi
     }
 
     @Override
-    public void create(final Measure payload) throws BanyanDBException {
-        execute(() ->
+    public long create(final Measure payload) throws BanyanDBException {
+        BanyandbDatabase.MeasureRegistryServiceCreateResponse resp = execute(() ->
                 stub.create(BanyandbDatabase.MeasureRegistryServiceCreateRequest.newBuilder()
                         .setMeasure(payload.serialize())
                         .build()));
+        return resp.getModRevision();
     }
 
     @Override
@@ -70,6 +71,15 @@ public class MeasureMetadataRegistry extends MetadataClient<MeasureRegistryServi
         return Measure.fromProtobuf(resp.getMeasure());
     }
 
+    @Override
+    public ResourceExist exist(String group, String name) throws BanyanDBException {
+        BanyandbDatabase.MeasureRegistryServiceExistResponse resp = execute(() ->
+                stub.exist(BanyandbDatabase.MeasureRegistryServiceExistRequest.newBuilder()
+                        .setMetadata(BanyandbCommon.Metadata.newBuilder().setGroup(group).setName(name).build())
+                        .build()));
+        return ResourceExist.create(resp.getHasGroup(), resp.getHasMeasure());
+    }
+
     @Override
     public List<Measure> list(final String group) throws BanyanDBException {
         BanyandbDatabase.MeasureRegistryServiceListResponse resp = execute(() ->
