@@ -18,16 +18,21 @@
 
 package org.apache.skywalking.banyandb.v1.client;
 
+import io.grpc.ServerServiceDefinition;
 import io.grpc.stub.StreamObserver;
 import org.apache.skywalking.banyandb.common.v1.BanyandbCommon;
 import org.apache.skywalking.banyandb.measure.v1.BanyandbMeasure;
 import org.apache.skywalking.banyandb.measure.v1.MeasureServiceGrpc;
+import org.apache.skywalking.banyandb.model.v1.BanyandbModel;
 import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException;
 import org.apache.skywalking.banyandb.v1.client.metadata.Duration;
 import org.apache.skywalking.banyandb.v1.client.metadata.IndexRule;
 import org.apache.skywalking.banyandb.v1.client.metadata.IndexRuleBinding;
 import org.apache.skywalking.banyandb.v1.client.metadata.Measure;
+import org.apache.skywalking.banyandb.v1.client.metadata.MeasureMetadataRegistry;
 import org.apache.skywalking.banyandb.v1.client.metadata.TagFamilySpec;
+
+import lombok.SneakyThrows;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
@@ -41,8 +46,11 @@ import java.util.List;
 import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 
+import static com.google.common.base.Preconditions.checkNotNull;
+
 public class BanyanDBClientMeasureWriteTest extends AbstractBanyanDBClientTest {
     private MeasureBulkWriteProcessor measureBulkWriteProcessor;
+    private Measure measure;
 
     @Before
     public void setUp() throws IOException, BanyanDBException {
@@ -51,17 +59,16 @@ public class BanyanDBClientMeasureWriteTest extends AbstractBanyanDBClientTest {
 
         measureBulkWriteProcessor = client.buildMeasureWriteProcessor(1000, 1, 1);
 
-        Measure m = Measure.create("sw_metric", "service_cpm_minute", Duration.ofHours(1))
+        measure = Measure.create("sw_metric", "service_cpm_minute", Duration.ofHours(1))
                 .setEntityRelativeTags("entity_id")
                 .addTagFamily(TagFamilySpec.create("default")
-                        .addIDTagSpec()
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("entity_id"))
                         .build())
                 .addField(Measure.FieldSpec.newIntField("total").compressWithZSTD().encodeWithGorilla().build())
                 .addField(Measure.FieldSpec.newIntField("value").compressWithZSTD().encodeWithGorilla().build())
                 .addIndex(IndexRule.create("scope", IndexRule.IndexType.INVERTED, IndexRule.IndexLocation.SERIES))
                 .build();
-        client.define(m);
+        client.define(measure);
     }
 
     @After
@@ -83,35 +90,12 @@ public class BanyanDBClientMeasureWriteTest extends AbstractBanyanDBClientTest {
         final List<BanyandbMeasure.WriteRequest> writeRequestDelivered = new ArrayList<>();
 
         // implement the fake service
-        final MeasureServiceGrpc.MeasureServiceImplBase serviceImpl =
-                new MeasureServiceGrpc.MeasureServiceImplBase() {
-                    @Override
-                    public StreamObserver<BanyandbMeasure.WriteRequest> write(StreamObserver<BanyandbMeasure.WriteResponse> responseObserver) {
-                        return new StreamObserver<BanyandbMeasure.WriteRequest>() {
-                            @Override
-                            public void onNext(BanyandbMeasure.WriteRequest value) {
-                                writeRequestDelivered.add(value);
-                                responseObserver.onNext(BanyandbMeasure.WriteResponse.newBuilder().build());
-                            }
-
-                            @Override
-                            public void onError(Throwable t) {
-                            }
-
-                            @Override
-                            public void onCompleted() {
-                                responseObserver.onCompleted();
-                                allRequestsDelivered.countDown();
-                            }
-                        };
-                    }
-                };
+        final ServerServiceDefinition serviceImpl = createService(allRequestsDelivered, writeRequestDelivered);
         serviceRegistry.addService(serviceImpl);
 
         Instant now = Instant.now();
-        MeasureWrite measureWrite = new MeasureWrite("sw_metric", "service_cpm_minute", now.toEpochMilli());
-        measureWrite.tag("id", TagAndValue.idTagValue("1"))
-                .tag("entity_id", TagAndValue.stringTagValue("entity_1"))
+        MeasureWrite measureWrite = client.createMeasureWrite("sw_metric", "service_cpm_minute", now.toEpochMilli());
+        measureWrite.tag("entity_id", TagAndValue.stringTagValue("entity_1"))
                 .field("total", TagAndValue.longFieldValue(100))
                 .field("value", TagAndValue.longFieldValue(1));
 
@@ -120,13 +104,108 @@ public class BanyanDBClientMeasureWriteTest extends AbstractBanyanDBClientTest {
         if (allRequestsDelivered.await(5, TimeUnit.SECONDS)) {
             Assert.assertEquals(1, writeRequestDelivered.size());
             final BanyandbMeasure.WriteRequest request = writeRequestDelivered.get(0);
-            Assert.assertEquals(2, request.getDataPoint().getTagFamilies(0).getTagsCount());
-            Assert.assertEquals("1", request.getDataPoint().getTagFamilies(0).getTags(0).getId().getValue());
-            Assert.assertEquals("entity_1", request.getDataPoint().getTagFamilies(0).getTags(1).getStr().getValue());
+            Assert.assertEquals(1, request.getDataPoint().getTagFamilies(0).getTagsCount());
+            Assert.assertEquals("entity_1", request.getDataPoint().getTagFamilies(0).getTags(0).getStr().getValue());
             Assert.assertEquals(100, request.getDataPoint().getFields(0).getInt().getValue());
             Assert.assertEquals(1, request.getDataPoint().getFields(1).getInt().getValue());
+            Assert.assertTrue(request.getMessageId() > 0);
         } else {
             Assert.fail();
         }
     }
+
+    @Test
+    public void testAutoRefreshSchema() throws Exception {
+        CountDownLatch allRequestsDelivered = new CountDownLatch(1);
+        List<BanyandbMeasure.WriteRequest> writeRequestDelivered = new ArrayList<>();
+        ServerServiceDefinition serviceImpl = createService(allRequestsDelivered, writeRequestDelivered);
+        serviceRegistry.addService(serviceImpl);
+
+        Instant now = Instant.now();
+        MeasureWrite measureWrite = client.createMeasureWrite("sw_metric", "service_cpm_minute", now.toEpochMilli());
+        measureWrite.tag("entity_id", TagAndValue.stringTagValue("entity_1"))
+                .field("total", TagAndValue.longFieldValue(100))
+                .field("value", TagAndValue.longFieldValue(1));
+
+        // update schema
+        Measure measureUpdate = measure.toBuilder()
+                .addField(Measure.FieldSpec.newIntField("new_field").compressWithZSTD().encodeWithGorilla().build())
+                .build();
+        MeasureMetadataRegistry measureMetadataRegistry = new MeasureMetadataRegistry(checkNotNull(this.channel));
+        measureMetadataRegistry.update(measureUpdate);
+
+        measureBulkWriteProcessor.add(measureWrite);
+        if (allRequestsDelivered.await(5, TimeUnit.SECONDS)) {
+            Assert.assertEquals(0, writeRequestDelivered.size());
+        } else {
+            Assert.fail();
+        }
+
+        // rewrite
+        serviceRegistry.removeService(serviceImpl);
+        allRequestsDelivered = new CountDownLatch(1);
+        writeRequestDelivered = new ArrayList<>();
+        serviceImpl = createService(allRequestsDelivered, writeRequestDelivered);
+        serviceRegistry.addService(serviceImpl);
+
+        now = Instant.now();
+        measureWrite = client.createMeasureWrite("sw_metric", "service_cpm_minute", now.toEpochMilli());
+        measureWrite.tag("entity_id", TagAndValue.stringTagValue("entity_1"))
+                .field("total", TagAndValue.longFieldValue(100))
+                .field("value", TagAndValue.longFieldValue(1));
+
+        measureBulkWriteProcessor.add(measureWrite);
+        if (allRequestsDelivered.await(5, TimeUnit.SECONDS)) {
+            Assert.assertEquals(1, writeRequestDelivered.size());
+            final BanyandbMeasure.WriteRequest request = writeRequestDelivered.get(0);
+            Assert.assertEquals(1, request.getDataPoint().getTagFamilies(0).getTagsCount());
+            Assert.assertEquals("entity_1", request.getDataPoint().getTagFamilies(0).getTags(0).getStr().getValue());
+            Assert.assertEquals(100, request.getDataPoint().getFields(0).getInt().getValue());
+            Assert.assertEquals(1, request.getDataPoint().getFields(1).getInt().getValue());
+            Assert.assertTrue(request.getMessageId() > 0);
+        } else {
+            Assert.fail();
+        }
+    }
+
+    @SneakyThrows
+    private boolean checkSchemaExpired(BanyandbMeasure.WriteRequest request) {
+        Measure m = client.findMeasure(request.getMetadata().getGroup(), request.getMetadata().getName());
+        return request.getMetadata().getModRevision() != m.serialize().getMetadata().getModRevision();
+    }
+
+    private ServerServiceDefinition createService(final CountDownLatch allRequestsDelivered,
+                                                  final List<BanyandbMeasure.WriteRequest> writeRequestDelivered) {
+        return new MeasureServiceGrpc.MeasureServiceImplBase() {
+            @Override
+            public StreamObserver<BanyandbMeasure.WriteRequest> write(StreamObserver<BanyandbMeasure.WriteResponse> responseObserver) {
+                return new StreamObserver<BanyandbMeasure.WriteRequest>() {
+                    @Override
+                    public void onNext(BanyandbMeasure.WriteRequest request) {
+                        if (checkSchemaExpired(request)) {
+                            responseObserver.onNext(
+                                    BanyandbMeasure.WriteResponse.newBuilder()
+                                            .setMetadata(request.getMetadata())
+                                            .setStatus(BanyandbModel.Status.STATUS_EXPIRED_SCHEMA)
+                                            .setMessageId(request.getMessageId())
+                                            .build());
+                        } else {
+                            writeRequestDelivered.add(request);
+                            responseObserver.onNext(BanyandbMeasure.WriteResponse.newBuilder().setMessageId(request.getMessageId()).build());
+                        }
+                    }
+
+                    @Override
+                    public void onError(Throwable t) {
+                    }
+
+                    @Override
+                    public void onCompleted() {
+                        responseObserver.onCompleted();
+                        allRequestsDelivered.countDown();
+                    }
+                };
+            }
+        }.bindService();
+    }
 }
