@@ -30,27 +30,26 @@ import java.io.IOException;
 public class BanyanDBClientTestCI {
     private static final String REGISTRY = "ghcr.io";
     private static final String IMAGE_NAME = "apache/skywalking-banyandb";
-    private static final String TAG = "a558725fb909e136694aeb2d879dd6a3ae13b6fc";
+    private static final String TAG = "f3abf75e66695f3c9fcae3138ad55e7a9d74f39a";
 
     private static final String IMAGE = REGISTRY + "/" + IMAGE_NAME + ":" + TAG;
 
-    protected static final int BANYANDB_PORT = 17912;
+    protected static final int GRPC_PORT = 17912;
+    protected static final int HTTP_PORT = 17913;
 
     @Rule
     public GenericContainer<?> banyanDB = new GenericContainer<>(
             DockerImageName.parse(IMAGE))
             .withCommand("standalone", "--stream-root-path", "/tmp/banyandb-stream-data",
                     "--measure-root-path", "/tmp/banyand-measure-data")
-            .withExposedPorts(BANYANDB_PORT)
-            .waitingFor(
-                    Wait.forLogMessage(".*Listening to\\*\\*\\*\\* addr::17912 module:LIAISON-GRPC\\n", 1)
-            );
+            .withExposedPorts(GRPC_PORT, HTTP_PORT)
+            .waitingFor(Wait.forHttp("/api/healthz").forPort(HTTP_PORT));
 
     protected BanyanDBClient client;
 
     protected void setUpConnection() throws IOException {
         log.info("create BanyanDB client and try to connect");
-        client = new BanyanDBClient(banyanDB.getHost(), banyanDB.getMappedPort(BANYANDB_PORT));
+        client = new BanyanDBClient(String.format("%s:%d", banyanDB.getHost(), banyanDB.getMappedPort(GRPC_PORT)));
         client.connect();
     }
 
