@@ -38,12 +38,12 @@ public class MeasureMetadataRegistryTest extends AbstractBanyanDBClientTest {
         Measure expectedMeasure = Measure.create("sw_metric", "service_cpm_minute", Duration.ofHours(1))
                 .setEntityRelativeTags("entity_id")
                 .addTagFamily(TagFamilySpec.create("default")
-                        .addIDTagSpec()
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("entity_id"))
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("scope"))
                         .build())
                 .addField(Measure.FieldSpec.newIntField("total").compressWithZSTD().encodeWithGorilla().build())
                 .addField(Measure.FieldSpec.newIntField("value").compressWithZSTD().encodeWithGorilla().build())
+                .addField(Measure.FieldSpec.newFloatField("sum").compressWithZSTD().encodeWithGorilla().build())
                 .addIndex(IndexRule.create("scope", IndexRule.IndexType.INVERTED, IndexRule.IndexLocation.SERIES))
                 .build();
         this.client.define(expectedMeasure);
@@ -52,6 +52,7 @@ public class MeasureMetadataRegistryTest extends AbstractBanyanDBClientTest {
         Assert.assertNotNull(actualMeasure);
         Assert.assertEquals(expectedMeasure, actualMeasure);
         Assert.assertNotNull(actualMeasure.updatedAt());
+        Assert.assertNotNull(actualMeasure.modRevision());
     }
 
     @Test
@@ -59,18 +60,19 @@ public class MeasureMetadataRegistryTest extends AbstractBanyanDBClientTest {
         Measure expectedMeasure = Measure.create("sw_metric", "service_cpm_minute", Duration.ofHours(1))
                 .setEntityRelativeTags("entity_id")
                 .addTagFamily(TagFamilySpec.create("default")
-                        .addIDTagSpec()
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("entity_id"))
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("scope"))
                         .build())
                 .addField(Measure.FieldSpec.newIntField("total").compressWithZSTD().encodeWithGorilla().build())
                 .addField(Measure.FieldSpec.newIntField("value").compressWithZSTD().encodeWithGorilla().build())
+                .addField(Measure.FieldSpec.newFloatField("sum").compressWithZSTD().encodeWithGorilla().build())
                 .addIndex(IndexRule.create("scope", IndexRule.IndexType.INVERTED, IndexRule.IndexLocation.SERIES))
                 .build();
         this.client.define(expectedMeasure);
         List<Measure> actualMeasures = new MeasureMetadataRegistry(this.channel).list("sw_metric");
         Assert.assertNotNull(actualMeasures);
         Assert.assertEquals(1, actualMeasures.size());
+        actualMeasures.forEach(measure -> Assert.assertNotNull(measure.modRevision()));
     }
 
     @Test
@@ -78,16 +80,16 @@ public class MeasureMetadataRegistryTest extends AbstractBanyanDBClientTest {
         Measure expectedMeasure = Measure.create("sw_metric", "service_cpm_minute", Duration.ofHours(1))
                 .setEntityRelativeTags("entity_id")
                 .addTagFamily(TagFamilySpec.create("default")
-                        .addIDTagSpec()
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("entity_id"))
                         .addTagSpec(TagFamilySpec.TagSpec.newStringTag("scope"))
                         .build())
                 .addField(Measure.FieldSpec.newIntField("total").compressWithZSTD().encodeWithGorilla().build())
                 .addField(Measure.FieldSpec.newIntField("value").compressWithZSTD().encodeWithGorilla().build())
+                .addField(Measure.FieldSpec.newFloatField("sum").compressWithZSTD().encodeWithGorilla().build())
                 .addIndex(IndexRule.create("scope", IndexRule.IndexType.INVERTED, IndexRule.IndexLocation.SERIES))
                 .build();
         this.client.define(expectedMeasure);
-        boolean deleted = new MeasureMetadataRegistry(this.channel).delete("sw_metric", "service_cpm_minute");
+        boolean deleted = this.client.delete(expectedMeasure);
         Assert.assertTrue(deleted);
         Assert.assertEquals(0, measureRegistry.size());
     }
