@@ -18,6 +18,7 @@
 
 package org.apache.skywalking.banyandb.v1.client.metadata;
 
+import com.google.common.base.Strings;
 import io.grpc.stub.StreamObserver;
 import org.apache.skywalking.banyandb.property.v1.BanyandbProperty;
 import org.apache.skywalking.banyandb.property.v1.PropertyServiceGrpc;
@@ -28,45 +29,46 @@ import org.apache.skywalking.banyandb.v1.client.util.TimeUtils;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.powermock.core.classloader.annotations.PowerMockIgnore;
-import org.powermock.modules.junit4.PowerMockRunner;
 
 import java.io.IOException;
 import java.time.ZonedDateTime;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 
 import static org.mockito.AdditionalAnswers.delegatesTo;
-import static org.powermock.api.mockito.PowerMockito.mock;
+import static org.mockito.Mockito.mock;
 
-@RunWith(PowerMockRunner.class)
-@PowerMockIgnore("javax.management.*")
 public class PropertyStoreTest extends AbstractBanyanDBClientTest {
     private PropertyStore store;
 
     private Map<String, BanyandbProperty.Property> memory;
 
+    private Set<Long> leasePool;
+
     private final PropertyServiceGrpc.PropertyServiceImplBase propertyServiceImpl = mock(PropertyServiceGrpc.PropertyServiceImplBase.class, delegatesTo(
             new PropertyServiceGrpc.PropertyServiceImplBase() {
                 @Override
-                public void create(BanyandbProperty.CreateRequest request, StreamObserver<BanyandbProperty.CreateResponse> responseObserver) {
+                public void apply(BanyandbProperty.ApplyRequest request, StreamObserver<BanyandbProperty.ApplyResponse> responseObserver) {
                     BanyandbProperty.Property p = request.getProperty().toBuilder()
                             .setUpdatedAt(TimeUtils.buildTimestamp(ZonedDateTime.now()))
                             .build();
+                    String key = format(p.getMetadata());
+                    BanyandbProperty.Property v = memory.get(key);
                     memory.put(format(p.getMetadata()), p);
-                    responseObserver.onNext(BanyandbProperty.CreateResponse.newBuilder().build());
-                    responseObserver.onCompleted();
-                }
-
-                @Override
-                public void update(BanyandbProperty.UpdateRequest request, StreamObserver<BanyandbProperty.UpdateResponse> responseObserver) {
-                    BanyandbProperty.Property p = request.getProperty().toBuilder()
-                            .setUpdatedAt(TimeUtils.buildTimestamp(ZonedDateTime.now()))
-                            .build();
-                    memory.put(format(p.getMetadata()), p);
-                    responseObserver.onNext(BanyandbProperty.UpdateResponse.newBuilder().build());
+                    BanyandbProperty.ApplyResponse.Builder builder = BanyandbProperty.ApplyResponse.newBuilder().setTagsNum(p.getTagsCount());
+                    if (!Strings.isNullOrEmpty(p.getTtl())) {
+                        long leaseId = System.currentTimeMillis();
+                        leasePool.add(leaseId);
+                        builder.setLeaseId(leaseId);
+                    }
+                    if (v == null) {
+                        responseObserver.onNext(builder.setCreated(true).build());
+                    } else {
+                        responseObserver.onNext(builder.setCreated(false).build());
+                    }
                     responseObserver.onCompleted();
                 }
 
@@ -89,21 +91,36 @@ public class PropertyStoreTest extends AbstractBanyanDBClientTest {
                     responseObserver.onNext(BanyandbProperty.ListResponse.newBuilder().addAllProperty(memory.values()).build());
                     responseObserver.onCompleted();
                 }
+
+                public void keepAlive(BanyandbProperty.KeepAliveRequest request, StreamObserver<BanyandbProperty.KeepAliveResponse> responseObserver) {
+                    if (!leasePool.contains(request.getLeaseId())) {
+                        responseObserver.onError(new RuntimeException("lease not found"));
+                    } else {
+                        responseObserver.onNext(BanyandbProperty.KeepAliveResponse.newBuilder().build());
+                    }
+                    responseObserver.onCompleted();
+                }
             }));
 
     @Before
     public void setUp() throws IOException {
         super.setUp(bindService(propertyServiceImpl));
         this.memory = new HashMap<>();
+        this.leasePool = new HashSet<>();
         this.store = new PropertyStore(this.channel);
     }
 
     @Test
-    public void testPropertyStore_create() throws BanyanDBException {
+    public void testPropertyStore_apply() throws BanyanDBException {
         Property property = Property.create("default", "sw", "ui_template")
                 .addTag(TagAndValue.newStringTag("name", "hello"))
                 .build();
-        this.store.create(property);
+        Assert.assertTrue(this.store.apply(property).created());
+        Assert.assertEquals(memory.size(), 1);
+        property = Property.create("default", "sw", "ui_template")
+                .addTag(TagAndValue.newStringTag("name", "hello1"))
+                .build();
+        Assert.assertFalse(this.store.apply(property).created());
         Assert.assertEquals(memory.size(), 1);
     }
 
@@ -112,7 +129,7 @@ public class PropertyStoreTest extends AbstractBanyanDBClientTest {
         Property property = Property.create("default", "sw", "ui_template")
                 .addTag(TagAndValue.newStringTag("name", "hello"))
                 .build();
-        this.store.create(property);
+        Assert.assertTrue(this.store.apply(property).created());
         Property gotProperty = this.store.get("default", "sw", "ui_template");
         Assert.assertNotNull(gotProperty);
         Assert.assertEquals(property, gotProperty);
@@ -124,8 +141,8 @@ public class PropertyStoreTest extends AbstractBanyanDBClientTest {
         Property property = Property.create("default", "sw", "ui_template")
                 .addTag(TagAndValue.newStringTag("name", "hello"))
                 .build();
-        this.store.create(property);
-        List<Property> listProperties = this.store.list("default", "sw");
+        Assert.assertTrue(this.store.apply(property, PropertyStore.Strategy.REPLACE).created());
+        List<Property> listProperties = this.store.list("default", "sw", null, null);
         Assert.assertNotNull(listProperties);
         Assert.assertEquals(1, listProperties.size());
         Assert.assertEquals(listProperties.get(0), property);
@@ -136,12 +153,24 @@ public class PropertyStoreTest extends AbstractBanyanDBClientTest {
         Property property = Property.create("default", "sw", "ui_template")
                 .addTag(TagAndValue.newStringTag("name", "hello"))
                 .build();
-        this.store.create(property);
-        boolean deleted = this.store.delete("default", "sw", "ui_template");
+        Assert.assertTrue(this.store.apply(property).created());
+        boolean deleted = this.store.delete("default", "sw", "ui_template").deleted();
         Assert.assertTrue(deleted);
         Assert.assertEquals(0, memory.size());
     }
 
+    @Test
+    public void testPropertyStore_keepAlive() throws BanyanDBException {
+        Property property = Property.create("default", "sw", "ui_template")
+                .addTag(TagAndValue.newStringTag("name", "hello"))
+                .setTtl("30m")
+                .build();
+        PropertyStore.ApplyResult resp = this.store.apply(property);
+        Assert.assertTrue(resp.created());
+        Assert.assertTrue(resp.leaseId() > 0);
+        this.store.keepAlive(resp.leaseId());
+    }
+
     static String format(BanyandbProperty.Metadata metadata) {
         return metadata.getContainer().getGroup() + ":" + metadata.getContainer().getName() + "/" + metadata.getId();
     }
