@@ -23,15 +23,10 @@ import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
-import org.junit.runner.RunWith;
-import org.powermock.core.classloader.annotations.PowerMockIgnore;
-import org.powermock.modules.junit4.PowerMockRunner;
 
 import java.io.IOException;
 import java.util.List;
 
-@RunWith(PowerMockRunner.class)
-@PowerMockIgnore("javax.management.*")
 public class StreamMetadataRegistryTest extends AbstractBanyanDBClientTest {
     @Before
     public void setUp() throws IOException {
@@ -59,6 +54,7 @@ public class StreamMetadataRegistryTest extends AbstractBanyanDBClientTest {
         Assert.assertNotNull(actualStream);
         Assert.assertEquals(expectedStream, actualStream);
         Assert.assertNotNull(actualStream.updatedAt());
+        Assert.assertNotNull(actualStream.modRevision());
     }
 
     @Test
@@ -79,6 +75,7 @@ public class StreamMetadataRegistryTest extends AbstractBanyanDBClientTest {
         List<Stream> actualStreams = new StreamMetadataRegistry(this.channel).list("default");
         Assert.assertNotNull(actualStreams);
         Assert.assertEquals(1, actualStreams.size());
+        actualStreams.forEach(stream -> Assert.assertNotNull(stream.modRevision()));
     }
 
     @Test
@@ -96,7 +93,7 @@ public class StreamMetadataRegistryTest extends AbstractBanyanDBClientTest {
                 .addIndex(IndexRule.create("trace_id", IndexRule.IndexType.INVERTED, IndexRule.IndexLocation.GLOBAL))
                 .build();
         this.client.define(expectedStream);
-        boolean deleted = new StreamMetadataRegistry(this.channel).delete("default", "sw");
+        boolean deleted = this.client.delete(expectedStream);
         Assert.assertTrue(deleted);
         Assert.assertEquals(0, streamRegistry.size());
     }
